﻿// Unicessing Hands サンプル
// Written by NISHIDA Ryota, http://dev.eyln.com

using UnityEngine;
using UnityEngine.VR;
using Unicessing;

// Oculus Touchの位置、姿勢を箱として表示します。
public class UnicessingHands : UGraphics
{
    Vector3 leftPos, rightPos;         // 左手、右手コントローラーの位置
    Quaternion leftRot, rightRot;      // 左手、右手コントローラーの姿勢

    // 初期化
    protected override void Setup()
    {
        rotateDegrees();     // 回転の単位をラジアンではなく「度」にする
        stroke(0, 255, 100); // 緑色のワイヤーフレームにする
    }

    // 毎フレームの更新と描画
    protected override void Draw()
    {
        UpdateHands();    // 手の位置を更新
        DrawHands();      // 手の位置に箱を描画
    }

    // 手の位置を更新
    void UpdateHands()
    {
        leftPos = InputTracking.GetLocalPosition(VRNode.LeftHand);   // 左手位置
        rightPos = InputTracking.GetLocalPosition(VRNode.RightHand); // 右手位置
        leftRot = InputTracking.GetLocalRotation(VRNode.LeftHand);   // 左手姿勢
        rightRot = InputTracking.GetLocalRotation(VRNode.RightHand); // 右手姿勢
    }

    // 手の位置に箱を描画
    void DrawHands()
    {
        float handSize = 0.15f;
        DrawHandBox(leftPos, leftRot, handSize);
        DrawHandBox(rightPos, rightRot, handSize);
    }

    // 手の箱を描画
    void DrawHandBox(Vector3 pos, Quaternion quat, float size)
    {
        pushMatrix();   // 現在位置、姿勢などを保存
        translate(pos); // 移動する
        rotate(quat.eulerAngles); // 回転する
        //uniTransfrom.rotation = quat; // rotate(quat.eulerAngles)のかわりにこうしてもよい
        noFill();       // 塗りつぶさない
        box(size);      // 箱を描く
        popMatrix();    // 現在位置、姿勢などを戻す
    }
}
